//
//  ViewController.swift
//  goong-sample-mapLibre-ios
//
//  Created by macos on 10/6/24.
//
import UIKit
import MapLibre
import CoreLocation
import Polyline

class ViewController: UIViewController,UITableViewDelegate, UITableViewDataSource,MLNMapViewDelegate {
    var mapView: MLNMapView!
    var locationManager = LocationManager()
    var searchViewModel = SearchViewModel()
    var placeDetailViewModel = PlaceDetailViewModel()
    var directionViewModel = DirectionViewModel()
    var coordinates = [Any]()
    var isActive = false {
        didSet {
            setupUIComponents()
        }
    }
    var isShowSatellite = false {
        didSet {
            setupMapView()
        }
    }
    private let textField = UITextField()
    private let tableView = UITableView()
    override func viewDidLoad() {
        super.viewDidLoad()
        setupMapView()
        setupUIComponents()
            
    }
    var isStyleLoaded = false
    
    private func setupMapView() {
        // create the map view
            mapView = MLNMapView(frame: view.bounds)
            mapView.autoresizingMask = [.flexibleWidth, .flexibleHeight]
            mapView.logoView.isHidden = true
            mapView.delegate = self
            mapView.styleURL = URL(string: isShowSatellite ? "https://tiles.goong.io/assets/goong_satellite.json?api_key=\(GoongConstants.API_KEY)" :"https://tiles.goong.io/assets/goong_map_web.json?api_key=\(GoongConstants.API_KEY)")
            view.addSubview(mapView)
            // Set the initial camera position
            self.mapViewCameraCurrent()
            
        self.setupExample() // Lan dau tien luc vao man
        // Gọi lại setupUIComponents để đảm bảo các UI components được thêm vào view
        self.setupUIComponents()
    }
    private func setupUIComponents() {
        // Create and configure the text input field
        textField.borderStyle = .roundedRect
        textField.placeholder = "Nhập địa chỉ"
        textField.translatesAutoresizingMaskIntoConstraints = false
        textField.addTarget(self, action: #selector(textInputDidChange(_:)), for: .editingChanged)
        
        // Create and configure the navigation button
        let navigationButton = UIButton(type: .system)
        navigationButton.setTitle("Dẫn đường", for: .normal)
        navigationButton.translatesAutoresizingMaskIntoConstraints = false
        navigationButton.backgroundColor = .blue
        navigationButton.setTitleColor(.white, for: .normal)
        navigationButton.layer.cornerRadius = 5
        navigationButton.layer.borderWidth = 1
        navigationButton.layer.borderColor = UIColor.blue.cgColor
        navigationButton.addTarget(self, action: #selector(navigationButtonTapped), for: .touchUpInside)
        
        // Create and configure the image button
        let imageButton = UIButton(type: .system)
        imageButton.translatesAutoresizingMaskIntoConstraints = false
        if let image = UIImage(named: "ic_map_type")?.withRenderingMode(.alwaysTemplate) {
            imageButton.setImage(image, for: .normal)
        }
        imageButton.tintColor = .black
        imageButton.addTarget(self, action: #selector(imageButtonTapped), for: .touchUpInside)
        
        // Add the text field to the view
        view.addSubview(textField)
        view.addSubview(navigationButton)
        view.addSubview(imageButton)
        
        // Setup Auto Layout constraints
        NSLayoutConstraint.activate([
            // Position the text field at the top of the view
            textField.topAnchor.constraint(equalTo: view.safeAreaLayoutGuide.topAnchor, constant: 20),
            textField.leadingAnchor.constraint(equalTo: view.leadingAnchor, constant: 20),
            textField.trailingAnchor.constraint(equalTo: view.trailingAnchor, constant: -100)
        ])
        
        NSLayoutConstraint.activate([
            // Position the navigation button next to the text field
            navigationButton.leadingAnchor.constraint(equalTo: textField.trailingAnchor, constant: 8),
            navigationButton.trailingAnchor.constraint(equalTo: view.trailingAnchor, constant: -10),
            navigationButton.centerYAnchor.constraint(equalTo: textField.centerYAnchor)
        ])
        
        NSLayoutConstraint.activate([
            imageButton.bottomAnchor.constraint(equalTo: view.safeAreaLayoutGuide.bottomAnchor, constant: -98),
            imageButton.trailingAnchor.constraint(equalTo: view.trailingAnchor, constant: -20),
            imageButton.widthAnchor.constraint(equalToConstant: 70),
            imageButton.heightAnchor.constraint(equalToConstant: 70)
        ])
        
        if isActive {
            // Ensure the table view is configured only once
            if tableView.superview == nil {
                // Configure the table view
                tableView.delegate = self
                tableView.dataSource = self
                tableView.translatesAutoresizingMaskIntoConstraints = false
                tableView.register(UITableViewCell.self, forCellReuseIdentifier: "PredictionCell")
                
                // Add the table view to the view
                view.addSubview(tableView)
                
                // Calculate the height constraint for the table view
                let tableViewHeight = view.frame.height / 3
                
                // Setup Auto Layout constraints for the table view
                NSLayoutConstraint.activate([
                    tableView.topAnchor.constraint(equalTo: textField.bottomAnchor, constant: 20),
                    tableView.leadingAnchor.constraint(equalTo: view.leadingAnchor, constant: 24),
                    tableView.trailingAnchor.constraint(equalTo: view.trailingAnchor, constant: -20),
                    tableView.heightAnchor.constraint(equalToConstant: tableViewHeight)
                ])
            }
        } else {
            // Remove the table view from the superview
            if tableView.superview != nil {
                tableView.removeFromSuperview()
            }
        }
    }
    
    @objc private func textInputDidChange(_ textField: UITextField) {
        guard let text = textField.text else { return }
        let currentLocation = CLLocationCoordinate2D(latitude: (self.locationManager.lastLocation?.coordinate.latitude) ?? Constants.latitudeSimulator, longitude: (self.locationManager.lastLocation?.coordinate.longitude) ?? Constants.longitudeSimulator)
        searchViewModel.ConvertURL(currentLocation:currentLocation, searchText: text)
        tableView.reloadData()
        self.isActive = true
    }
    @objc func navigationButtonTapped() {
        if let locationCurrent = locationManager.lastLocation{
            guard let destinationLat = self.placeDetailViewModel.locationDetail?.result.geometry.location.lat,
                  let destinationLng = self.placeDetailViewModel.locationDetail?.result.geometry.location.lng else {
                print("Error: Invalid destination coordinates")
                return
            }
            directionViewModel.ConvertURL(
                origin: "\(locationCurrent.coordinate.latitude),\(locationCurrent.coordinate.longitude)",
                destination: "\(destinationLat),\(destinationLng)"
            )
        }else{
            guard let destinationLat = self.placeDetailViewModel.locationDetail?.result.geometry.location.lat,
                  let destinationLng = self.placeDetailViewModel.locationDetail?.result.geometry.location.lng else {
                print("Error: Invalid destination coordinates")
                return
            }
            directionViewModel.ConvertURL(
                origin: "\(Constants.latitudeSimulator),\(Constants.longitudeSimulator)",
                destination: "\(destinationLat),\(destinationLng)"
            )
        }
    }
    @objc private func imageButtonTapped() {
        // Handle the image button tap action
        self.isShowSatellite.toggle()
    }
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return searchViewModel.predictions?.predictions.count ?? 0
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "PredictionCell", for: indexPath)
        
        // Safely unwrap predictions and check if indexPath.row is within bounds
        if let predictions = searchViewModel.predictions?.predictions, indexPath.row < predictions.count {
            let prediction = predictions[indexPath.row]
            let mainText = prediction.description
            let distanceKm = prediction.distanceMeters / 1000
            cell.textLabel?.text = "\(mainText) - \(distanceKm) km"
        } else {
            // Handle the case where there are no predictions or the index is out of range
            cell.textLabel?.text = "No data available"
        }
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if let prediction = searchViewModel.predictions?.predictions[indexPath.row] {
            actionSelectLocation(place_id: prediction.placeID, description: prediction.description, locationTitle: prediction.structuredFormatting.mainText, distanceMeters: prediction.distanceMeters)
            
        }
    }
    
    private func actionSelectLocation(place_id: String, description: String, locationTitle: String, distanceMeters: Int) {
        self.isActive = false
        placeDetailViewModel.ConvertURL(placeID: place_id)
    }
    internal func setupExample() {
        // Lan dau tien luc vao man
                self.updateMapView()

                // Moi khi location update
                self.placeDetailViewModel.onLocationUpdate = {
                    self.updateMapView()
                    
                    guard let locationDetail = self.placeDetailViewModel.locationDetail else {
                        return
                    }
                    
                    let lat = locationDetail.result.geometry.location.lat
                    let lng = locationDetail.result.geometry.location.lng
                    
                    // Gọi hàm addImageToMap nếu style đã tải xong
                    if self.isStyleLoaded {
                        self.addDestinationMarker(latitude: lat, longitude: lng)
                    }
                    self.mapViewCameraDestination()
                }

                // Thêm marker cho vị trí hiện tại
                
                
                
                // Gọi hàm addImageToMap nếu style đã tải xong
                if self.isStyleLoaded {
                    self.addCurrentMarker(
                        latitude: self.locationManager.lastLocation?.coordinate.latitude ?? Constants.latitudeSimulator,
                        longitude: self.locationManager.lastLocation?.coordinate.longitude ?? Constants.longitudeSimulator
                    )
                }
                self.mapViewCameraCurrent()
    }
    
    func updateMapView (){
        self.directionViewModel.onDirectionUpdate = {
            guard let overviewPolylinePoints = self.directionViewModel.direction.routes.first?.overviewPolyline.points else {
                print("Error: overviewPolyline.points is nil")
                return
            }
            let polyline = Polyline(encodedPolyline: overviewPolylinePoints )
            //            let distance = Int(self.directionViewModel.direction.routes.first!.distance)
            
            let decodedCoordinates: [CLLocationCoordinate2D]? = polyline.coordinates
            decodedCoordinates?.forEach{
                var waypoints = [Any]()
                waypoints.append($0.longitude)
                waypoints.append($0.latitude)
                self.coordinates.append(waypoints)
            }
            self.addLine(decodedCoordinates:decodedCoordinates!)
        }
        self.removeAnnotationPoint()
        self.removeLineLayer()
    }
    func mapViewCameraDestination () {
        // Set the map’s center coordinate and zoom level.
        mapView.setCenter(
            CLLocationCoordinate2D(latitude: (self.placeDetailViewModel.locationDetail?.result.geometry.location.lat)!, longitude: (self.placeDetailViewModel.locationDetail?.result.geometry.location.lng)!),
            zoomLevel: 13,
            animated: true)
    }
    func addDestinationMarker(latitude: Double, longitude: Double) {
        guard let style = mapView.style else { return }
        
        // Add the destination marker image to the style
        if let image = UIImage(named: Constants.destination_marker) {
            style.setImage(image, forName: Constants.red_icon_id)
        }
        
        // Create a feature with the destination coordinates
        let coordinate = CLLocationCoordinate2D(latitude: latitude, longitude: longitude)
        let pointFeature = MLNPointFeature()
        pointFeature.coordinate = coordinate
        pointFeature.attributes = [Constants.icon_key_des: Constants.red_marker_property]
        
        // Create a shape source and add it to the style
        let source = MLNShapeSource(identifier: Constants.source_id_destination, shape: pointFeature, options: nil)
        style.addSource(source)
        
        // Create a symbol style layer and configure it
        let layer = MLNSymbolStyleLayer(identifier: Constants.layer_id_des, source: source)
        layer.iconImageName = NSExpression(forConstantValue: Constants.red_icon_id)
        layer.iconAllowsOverlap = NSExpression(forConstantValue: NSNumber(value: false))
        layer.iconScale = NSExpression(forConstantValue: NSNumber(value: 0.1))
        
        // Add the layer to the map's style
        style.addLayer(layer)
    }
        // Gọi hàm addImageToMap khi style đã tải xong
        func mapView(_ mapView: MLNMapView, didFinishLoading style: MLNStyle) {
            isStyleLoaded = true
            self.addCurrentMarker(
                latitude: self.locationManager.lastLocation?.coordinate.latitude ?? Constants.latitudeSimulator ,
                longitude: self.locationManager.lastLocation?.coordinate.longitude ?? Constants.longitudeSimulator )
        }
    func addCurrentMarker(latitude: Double, longitude: Double) {
        
            // Add the destination marker image to the style
            if let image = UIImage(named: Constants.ic_current_point) {
                mapView.style?.setImage(image, forName: Constants.blue_icon_id)
            }

            // Create a feature with the destination coordinates
            let coordinate = CLLocationCoordinate2D(latitude: latitude, longitude: longitude)
            let pointFeature = MLNPointFeature()
       
            pointFeature.coordinate = coordinate
            pointFeature.attributes = [Constants.icon_key: Constants.blue_marker_property]
            // Create a shape source and add it to the style
            let source = MLNShapeSource(identifier: Constants.source_id, shape: pointFeature, options: nil)
            mapView.style?.addSource(source)

            // Create a symbol style layer and configure it
            let layer = MLNSymbolStyleLayer(identifier: Constants.layer_id, source: source)
            layer.iconImageName = NSExpression(forConstantValue: Constants.blue_icon_id)
            layer.iconAllowsOverlap = NSExpression(forConstantValue: NSNumber(value: false))
            layer.iconScale = NSExpression(forConstantValue: NSNumber(value: 0.6))
            // Add the layer to the map's style
            mapView.style?.addLayer(layer)
        }
    
    func mapViewCameraCurrent() {
        // Set the map’s center coordinate and zoom level.

        let latitude = self.locationManager.lastLocation?.coordinate.latitude ?? Constants.latitudeSimulator
        let longitude = self.locationManager.lastLocation?.coordinate.longitude ?? Constants.longitudeSimulator
        
        // Đặt vị trí trung tâm và mức độ phóng to của mapView
        mapView.centerCoordinate = CLLocationCoordinate2D(latitude: latitude, longitude: longitude)
        mapView.zoomLevel = 15
    }
    func addLine(decodedCoordinates: [CLLocationCoordinate2D]) {
        let sourceIdentifier = Constants.lineSource
        
        if let existingSource = mapView.style?.source(withIdentifier: sourceIdentifier) as? MLNShapeSource {
            // Update the existing source with new data
            let polyline = MLNPolyline(coordinates: decodedCoordinates, count: UInt(decodedCoordinates.count))
            existingSource.shape = polyline
        } else {
            // Create a polyline with the coordinates
            let polyline = MLNPolyline(coordinates: decodedCoordinates, count: UInt(decodedCoordinates.count))
            
            // Create a GeoJSON source
            let source = MLNShapeSource(identifier: sourceIdentifier, shape: polyline, options: nil)
            mapView.style?.addSource(source)
            
            // Create a line layer and add it to the map
            let lineLayer = MLNLineStyleLayer(identifier: Constants.LINE_LAYER, source: source)
            lineLayer.lineColor = NSExpression(forConstantValue: UIColor.blue)
            lineLayer.lineWidth = NSExpression(forConstantValue: 10)
            lineLayer.lineCap = NSExpression(forConstantValue: "round")
            lineLayer.lineJoin = NSExpression(forConstantValue: "round")
            
            mapView.style?.addLayer(lineLayer)
        }
    }
    func removeAnnotationPoint() {
        guard let style = mapView.style else {
            print("Style chưa được tải.")
            return
        }
        
        // Kiểm tra và xóa layer nếu tồn tại
        if let layer = style.layer(withIdentifier: Constants.layer_id_des) {
            do {
                try style.removeLayer(layer)
            } catch {
                print("Lỗi khi xóa layer: \(error)")
            }
        } else {
            print("Layer không tồn tại.")
        }
        
        // Kiểm tra và xóa source nếu tồn tại
        if let source = style.source(withIdentifier: Constants.source_id_destination) {
            do {
                try style.removeSource(source)
            } catch {
                print("Lỗi khi xóa source: \(error)")
            }
        } else {
            print("Source không tồn tại.")
        }
    }
    func removeLineLayer (){
        guard let style = mapView.style else {
            print("Style chưa được tải.")
            return
        }
        
        // Kiểm tra và xóa layer nếu tồn tại
        if let layer = style.layer(withIdentifier: Constants.LINE_LAYER) {
            do {
                try style.removeLayer(layer)
            } catch {
                print("Lỗi khi xóa layer: \(error)")
            }
        } else {
            print("Layer không tồn tại.")
        }
        
        // Kiểm tra và xóa source nếu tồn tại
        if let source = style.source(withIdentifier: Constants.lineSource) {
            do {
                try style.removeSource(source)
            } catch {
                print("Lỗi khi xóa source: \(error)")
            }
        } else {
            print("Source không tồn tại.")
        }
    }
}

